﻿using System;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

using System.Data;
using System.Collections.Generic;
using System.Configuration;
using System.Web.Security;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Telerik.Web.UI;

public partial class RadSchedulerWebForm : System.Web.UI.Page 
{


    bool CreatingExceptionAppointment
    {
        get { return (bool)ViewState["CreatingExceptionAppointment"]; }
        set { ViewState["CreatingExceptionAppointment"] = value; }
    }

    Appointment EditedAppointment
    {
        get
        {
            return (EditedAppointmentID != null) ? RadScheduler1.Appointments.FindByID(EditedAppointmentID) : null;
        }
        set
        {
            EditedAppointmentID = value.ID;
            EditedAppointmentParentID = value.RecurrenceParentID;
        }
    }

    Appointment EditedAppointmentParent
    {
        get
        {
            return (EditedAppointmentParentID != null) ? RadScheduler1.Appointments.FindByID(EditedAppointmentParentID) : null;
        }
    }

    private object EditedAppointmentID
    {
        get { return ViewState["EditedAppointmentID"]; }
        set { ViewState["EditedAppointmentID"] = value; }
    }

    private object EditedAppointmentParentID
    {
        get { return ViewState["EditedAppointmentParentID"]; }
        set { ViewState["EditedAppointmentParentID"] = value; }
    }

    protected void RadScheduler1_FormCreating(object sender, SchedulerFormCreatingEventArgs e)
    {
        RadSchedulerRecurrenceEditor1.ResetLayout();

        if (e.Mode == SchedulerFormMode.Insert || e.Mode == SchedulerFormMode.Edit)
        {
            EditedAppointment = e.Appointment;
            e.Cancel = true;
        }

        var appointmentToEdit = RadScheduler1.PrepareToEdit(e.Appointment, RadScheduler1.EditingRecurringSeries);

        ScriptManager.RegisterStartupScript(Page, GetType(), "formScript", "Sys.Application.add_load(openForm);", true);
        PopulateEditForm(appointmentToEdit);
    }

    private bool IsAllDayAppointment(Appointment appointment)
    {
        DateTime displayStart = RadScheduler1.UtcToDisplay(appointment.Start);
        DateTime displayEnd = RadScheduler1.UtcToDisplay(appointment.End);
        return displayStart.CompareTo(displayStart.Date) == 0 && displayEnd.CompareTo(displayEnd.Date) == 0 && displayStart.CompareTo(displayEnd) != 0;
    }

    private DateTime Start
    {
        get
        {
            DateTime result = StartTime.SelectedDate.Value.Date;

            TimeSpan time = StartTime.SelectedDate.Value.TimeOfDay;
            result = result.Add(time);

            return RadScheduler1.DisplayToUtc(result);
        }
    }

    private DateTime End
    {
        get
        {
            DateTime result = EndTime.SelectedDate.Value.Date;

            TimeSpan time = EndTime.SelectedDate.Value.TimeOfDay;
            result = result.Add(time);

            return RadScheduler1.DisplayToUtc(result);
        }
    }

    private string RecurrenceRuleText
    {
        get
        {
            if (RadScheduler1.RecurrenceSupport)
            {
                RadSchedulerRecurrenceEditor1.StartDate = Start;
                RadSchedulerRecurrenceEditor1.EndDate = End;

                RecurrenceRule rrule = RadSchedulerRecurrenceEditor1.RecurrenceRule;

                if (rrule == null)
                {
                    return string.Empty;
                }

                RecurrenceRule originalRule;
                if (RecurrenceRule.TryParse(_originalRecurrenceRule.Value, out originalRule))
                {
                    rrule.Exceptions = originalRule.Exceptions;
                }

                if (rrule.Range.RecursUntil != DateTime.MaxValue)
                {
                    rrule.Range.RecursUntil = RadScheduler1.DisplayToUtc(rrule.Range.RecursUntil);
                }

                return rrule.ToString();
            }

            return string.Empty;
        }
        set
        {
            RecurrenceRule rrule = null;

            RecurrenceRule.TryParse(value, out rrule);

            if (rrule != null)
            {
                if (rrule.Range.RecursUntil != DateTime.MaxValue)
                {
                    DateTime recursUntil = RadScheduler1.UtcToDisplay(rrule.Range.RecursUntil);

                    if (!IsAllDayAppointment(EditedAppointment))
                    {
                        recursUntil = recursUntil.AddDays(-1);
                    }

                    rrule.Range.RecursUntil = recursUntil;
                }
            }
            RadSchedulerRecurrenceEditor1.RecurrenceRuleText = (rrule != null) ? rrule.ToString() : value;

            _originalRecurrenceRule.Value = value;
        }
    }

    private void PopulateEditForm(Appointment editedAppointment)
    {
        Appointment appointmentToEdit = RadScheduler1.PrepareToEdit(editedAppointment, RadScheduler1.EditingRecurringSeries);
        DescriptionText.Text = appointmentToEdit.Subject;

        StartTime.SelectedDate = RadScheduler1.UtcToDisplay(appointmentToEdit.Start);
        EndTime.SelectedDate = RadScheduler1.UtcToDisplay(appointmentToEdit.End);

        if ((appointmentToEdit.Reminders.Count == 0) || (ReminderDropDown.SelectedValue == ""))
        {
            ReminderDropDown.SelectedValue = "";
        }
        else
        {
            ReminderDropDown.SelectedValue = appointmentToEdit.Reminders[0].Trigger.TotalMinutes.ToString();
        }

        Resource user = appointmentToEdit.Resources.GetResourceByType("User");
        if (user != null)
        {
            UserDropDown.SelectedValue = user.Key.ToString();
        }

        RadSchedulerRecurrenceEditor1.StartDate = appointmentToEdit.Start;
        RadSchedulerRecurrenceEditor1.EndDate = appointmentToEdit.End;

        RecurrenceRuleText = appointmentToEdit.RecurrenceRule;
    }

    protected void SubmitButton_Click(object sender, EventArgs e)
    {
        if (EditedAppointment == null)
        {
            // Insert Appointment
            Appointment aptToInsert = PopulateBasicAppointmentPropertiesFromForm();

            RadScheduler1.InsertAppointment(aptToInsert);
        }
        else
        {
            if (!RadScheduler1.EditingRecurringSeries && (EditedAppointmentParent != null || EditedAppointment.RecurrenceState == RecurrenceState.Master))
            {
                // Create Exception Appointment
                var aptOccurence = EditedAppointment;
                var aptException = PopulateBasicAppointmentPropertiesFromForm(
                                    RadScheduler1.PrepareToEdit(aptOccurence, RadScheduler1.EditingRecurringSeries)
                );

                RadScheduler1.UpdateAppointment(aptException);
            }
            else
            {
                // Update Appointment
                Appointment aptOriginal = EditedAppointment;

                if (RadScheduler1.EditingRecurringSeries && (aptOriginal.RecurrenceState == RecurrenceState.Occurrence || aptOriginal.RecurrenceState == RecurrenceState.Exception))
                    aptOriginal = EditedAppointmentParent;

                Appointment aptToUpdate = PopulateBasicAppointmentPropertiesFromForm(aptOriginal.Clone());

                RadScheduler1.UpdateAppointment(aptToUpdate, aptOriginal);
            }
        }

        EditedAppointmentID = "";
        EditedAppointmentParentID = "";

        RadDock1.Closed = true;
    }

    private Appointment PopulateBasicAppointmentPropertiesFromForm()
    {
        return PopulateBasicAppointmentPropertiesFromForm(null);
    }

    private Appointment PopulateBasicAppointmentPropertiesFromForm(Appointment apt)
    {
        if (apt == null)
            apt = new Appointment();

        Resource user = new Resource("User", int.Parse(UserDropDown.SelectedValue), UserDropDown.SelectedItem.Text);
        DateTime start = RadScheduler1.DisplayToUtc(StartTime.SelectedDate.Value);
        DateTime end = RadScheduler1.DisplayToUtc(EndTime.SelectedDate.Value);
        string reminder = ReminderDropDown.SelectedValue;

        apt.Subject = DescriptionText.Text;
        apt.Start = start;
        apt.End = end;

        apt.RecurrenceRule = RecurrenceRuleText;

        if (!String.IsNullOrEmpty(reminder))
        {
            apt.Reminders.Add(new Reminder(int.Parse(reminder)));
        }

        Resource existingUser = apt.Resources.GetResourceByType("User");
        if (existingUser != null)
        {
            apt.Resources.Remove(existingUser);
        }
        apt.Resources.Add(user);

        return apt;
    }
   

  
}
